import argparse
import subprocess
from pathlib import Path
import os
import sys
from typing import List, Union, Any

import humanize

from fds.logger import Logger


def get_size_of_path(path: str) -> int:
    if os.path.isdir(path):
        return sum(p.stat().st_size for p in Path(path).rglob('*'))
    else:
        return os.stat(path).st_size


def convert_bytes_to_readable(bytes: int) -> str:
    return humanize.naturalsize(bytes)


def convert_bytes_to_string(bytes_data: bytes) -> str:
    return bytes_data.decode("utf-8")


def execute_command(command: Union[str, List[str]], shell: bool = False, capture_output: bool=True,
                    ignorable_return_codes: List[int] = [0]) -> Any:
    if capture_output:
        # capture_output is not available in python 3.6, so using PIPE manually
        output = subprocess.run(command, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    else:
        output = subprocess.run(command, shell=shell)
    if output.stderr is None or output.stdout is None:
        return
    logger = Logger.get_logger("fds")
    error_message = convert_bytes_to_string(output.stderr)
    if error_message != '':
        logger.error(error_message)
    if output.returncode not in ignorable_return_codes:
        raise Exception(error_message)
    return output


def append_line_to_file(filename: str, data: str) -> None:
    with open(filename, "a") as f:
        f.write(data)
        if not data.endswith('\n'):
            f.write('\n')


def does_file_exist(filename: str) -> bool:
    try:
        import os.path
        return os.path.exists(filename)
    except Exception as e:
        return False


def check_git_ignore(filename: str) -> Any:
    # You can ignore return code 1 too here, because it shows that the file is not ignored
    # return code 0 is when file is ignored
    git_output = execute_command(["git", "check-ignore", filename], capture_output=True, ignorable_return_codes=[0, 1])
    return git_output


def check_dvc_ignore(filename: str) -> Any:
    # You can ignore return code 1 too here, because it shows that the file is not ignored
    # return code 0 is when file is ignored
    git_output = execute_command(["dvc", "check-ignore", filename], capture_output=True, ignorable_return_codes=[0, 1])
    return git_output



from fds.utils import does_file_exist, execute_command, convert_bytes_to_string
from tests.it.helpers import IntegrationTestCase


class TestGit(IntegrationTestCase):

    def test_init_git(self):
        self.git_service.init()
        assert does_file_exist(f"{self.repo_path}/.git") is True

    def test_init_git_already_exists(self):
        execute_command(["git", "init"])
        assert does_file_exist(f"{self.repo_path}/.git") is True
        msg = self.git_service.init()
        assert msg == "git already initialized"

    def test_status(self):
        self.git_service.status()

    def test_add(self):
        self.git_service.init()
        super().create_fake_git_data()
        self.git_service.add(".")
        output = execute_command(["git", "status"], capture_output=True)
        assert convert_bytes_to_string(output.stderr) == ""
        assert f"new file:   git_data/file-0" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-1" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-2" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-3" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-4" in convert_bytes_to_string(output.stdout)

    def test_add_one(self):
        self.git_service.init()
        super().create_fake_git_data()
        self.git_service.add("git_data/file-0")
        output = execute_command(["git", "status"], capture_output=True)
        assert convert_bytes_to_string(output.stderr) == ""
        assert f"new file:   git_data/file-0" in convert_bytes_to_string(output.stdout)
        assert "Untracked files:" in convert_bytes_to_string(output.stdout)
        assert "file-1" in convert_bytes_to_string(output.stdout)
        assert "file-2" in convert_bytes_to_string(output.stdout)
        assert "file-3" in convert_bytes_to_string(output.stdout)
        assert "file-4" in convert_bytes_to_string(output.stdout)

    def test_add_gitignore(self):
        self.git_service.init()
        super().create_fake_git_data()
        super().create_dummy_file(".gitignore", 100)
        self.git_service.add("git_data/file-0")
        output = execute_command(["git", "status"], capture_output=True)
        assert f"new file:   git_data/file-0" in convert_bytes_to_string(output.stdout)
        assert f"new file:   .gitignore" in convert_bytes_to_string(output.stdout)

    def test_commit(self):
        self.git_service.init()
        super().create_fake_git_data()
        self.git_service.add(".")
        self.git_service.commit("Commit 1")
        output = execute_command(["git", "log", "--oneline"], capture_output=True)
        assert "Commit 1" in convert_bytes_to_string(output.stdout)




import os
from enum import Enum
from typing import Any, List, Optional, Tuple
import PyInquirer
from progress.bar import Bar

from fds.domain.commands import AddCommands
from fds.domain.constants import MAX_THRESHOLD_SIZE
from fds.logger import Logger
from fds.services.base_service import BaseService
from fds.services.pretty_print import PrettyPrint
from fds.utils import get_size_of_path, convert_bytes_to_readable, convert_bytes_to_string, execute_command, \
    append_line_to_file, check_git_ignore, check_dvc_ignore, does_file_exist


# Choices for DVC
class DvcChoices(Enum):
    ADD_TO_DVC = "Add to DVC"
    ADD_TO_GIT = "Add to Git"
    IGNORE = "Ignore"
    STEP_INTO = "Step Into"


class DVCService(BaseService):
    """
    DVC Service responsible for all the dvc commands of fds
    """

    def __init__(self):
        self.repo_path = os.path.curdir
        self.logger = Logger.get_logger("fds.DVCService")
        self.printer = PrettyPrint()
        self.selection_message_count = 0

    def init(self):
        """
        Responsible for running dvc init
        :return:
        """
        # Check if dvc is already initialized
        if does_file_exist(f"{self.repo_path}/.dvc"):
            return "DVC already initialized"
        execute_command(["dvc", "init", "--subdir"])
        return "DVC initialized successfully"

    def status(self) -> Any:
        """
        Responsible for running dvc status
        :return:
        """
        return execute_command(["dvc", "status"], capture_output=False)

    @staticmethod
    def __should_skip_list_add(directory: str) -> bool:
        """
        Check if the given dir should be skipped or not
        :param directory: the name of the dir
        :return: True if we should skip, else return False
        """
        if directory == ".":
            return True
        git_output = check_git_ignore(directory)
        if convert_bytes_to_string(git_output.stdout) != '':
            return True
        dvc_output = check_dvc_ignore(directory)
        if convert_bytes_to_string(dvc_output.stdout) != '':
            return True
        return False

    def __skip_already_added(self, root, dirs) -> None:
        # Check if current file is git ignored (this is very similar to adding to dvc also,
        # because as soon as we add to dvc it gets ignored)
        for d in dirs:
            directory = f"{root}/{d}"
            if self.__should_skip_list_add(directory):
                dirs.remove(d)

    @staticmethod
    def _get_choice(file_or_dir_to_check: str, path_size: int, file_dir_type: str) -> dict:
        choices = [{
            "key": "d",
            "name": "Add to DVC",
            "value": DvcChoices.ADD_TO_DVC.value
        }, {
            "key": "g",
            "name": "Add to Git",
            "value": DvcChoices.ADD_TO_GIT.value
        }, {
            "key": "i",
            "name": "Ignore - Add to .gitignore",
            "value": DvcChoices.IGNORE.value
        }]
        if os.path.isdir(file_or_dir_to_check):
            choices.append({
                "key": "s",
                "name": "Step Into",
                "value": DvcChoices.STEP_INTO.value
            })

        questions = [
            {
                "type": "expand",
                "message": f"What would you like to do with {file_dir_type} {file_or_dir_to_check} of "
                           f"{convert_bytes_to_readable(path_size)}?",
                "name": "selection_choice",
                "choices": choices,
                "default": DvcChoices.ADD_TO_DVC.value
            }
        ]
        answers = PyInquirer.prompt(questions)
        return answers

    def __get_to_add_to_dvc(self,
                            file_or_dir_to_check: str,
                            dirs: List[str],
                            file_dir_type: str) -> Tuple[Optional[str], Optional[str]]:
        """
        Returns the tuple (file/folder to be added to dvc, folder to be ignored)
        :param file_or_dir_to_check: File or folder to check if its to be added or ignored
        :param dirs: folders in the current walk
        :param file_dir_type: Type indicating whether its file or Dir
        :return: (file/folder to be added to dvc, folder to be ignored)
        """
        if not self.__should_skip_list_add(file_or_dir_to_check):
            path_size = get_size_of_path(file_or_dir_to_check)
            # Dont need to traverse deep in case of dir, if the dir is below the threshold size
            if path_size < MAX_THRESHOLD_SIZE:
                if os.path.isdir(file_or_dir_to_check):
                    return None, file_or_dir_to_check
                return None, None
            # Show the message only when files are shown and only once per add
            if self.selection_message_count == 0:
                self.selection_message_count = 1
                self.printer.warn('========== Make your selection, Press "h" for help ==========')
            answers = DVCService._get_choice(file_or_dir_to_check=file_or_dir_to_check,
                                             path_size=path_size,
                                             file_dir_type=file_dir_type)
            if answers["selection_choice"] == DvcChoices.ADD_TO_DVC.value:
                # Dont need to traverse deep
                [dirs.remove(d) for d in list(dirs)]
                return file_or_dir_to_check, None
            elif answers["selection_choice"] == DvcChoices.ADD_TO_GIT.value:
                # Dont need to traverse deep
                [dirs.remove(d) for d in list(dirs)]
                return None, None
            elif answers["selection_choice"] == DvcChoices.IGNORE.value:
                # We should ignore the ./ in beginning when adding to gitignore
                # Add files to gitignore
                append_line_to_file(".gitignore",
                                    file_or_dir_to_check[file_or_dir_to_check.startswith('./') and 2:])
                # Dont need to traverse deep
                [dirs.remove(d) for d in list(dirs)]
                return None, None
        return None, None

    def __add(self, add_argument: str):
        chosen_files_or_folders = []
        # Keep track of dirs which are below threshold size, so we dont iterate the files inside these dirs
        skipped_dirs = []
        # May be add all the folders given in the .gitignore
        folders_to_exclude = ['.git', '.dvc']
        if add_argument == AddCommands.ALL.value:
            path_to_walk = self.repo_path
        else:
            path_to_walk = f"{self.repo_path}/{add_argument}"
        # if argument is to add a file
        if os.path.isfile(path_to_walk) and get_size_of_path(path_to_walk) >= MAX_THRESHOLD_SIZE:
            # Keep the file in chosen list
            chosen_files_or_folders = [path_to_walk]
        for (root, dirs, files) in os.walk(path_to_walk, topdown=True, followlinks=False):
            # Now skip the un-necessary folders
            [dirs.remove(d) for d in list(dirs) if d in folders_to_exclude]
            # Skip the already added files/folders
            self.__skip_already_added(root, dirs)
            # First check root
            (folder_to_add, skipped_dir) = self.__get_to_add_to_dvc(root, dirs, "Dir")
            if skipped_dir is not None:
                skipped_dirs.append(skipped_dir)
            if folder_to_add is not None:
                chosen_files_or_folders.append(folder_to_add)
            else:
                # Only if they dont select the directory then ask for files,
                # otherwise ignore asking about files of the directory
                # We are also showing if the user chooses to skip because the user
                # might not know there is a large file in the directory and choose skip
                # because he doesn't want the entire directory to be added.

                # If the root is skipped because it is below threshold size then we don't need to check files
                if root in skipped_dirs:
                    continue
                # Then check files
                for file in files:
                    (file_to_add, skipped_dir) = self.__get_to_add_to_dvc(f"{root}/{file}", [], "File")
                    if skipped_dir is not None:
                        skipped_dirs.append(skipped_dir)
                    if file_to_add is not None:
                        chosen_files_or_folders.append(file_to_add)
        self.logger.debug(f"Chosen folders to be added to dvc are {chosen_files_or_folders}")
        if len(chosen_files_or_folders) == 0:
            return "Nothing to add in DVC"

        self.printer.warn("Adding to dvc...")
        progress_tracker = Bar('Processing', max=len(chosen_files_or_folders))
        for add_to_dvc in chosen_files_or_folders:
            execute_command(["dvc", "add", add_to_dvc])
            progress_tracker.next()
        progress_tracker.finish()
        return "DVC add successfully executed"

    def add(self, add_argument: str) -> Any:
        return self.__add(add_argument)

    def commit(self, auto_confirm: bool) -> Any:
        """
        Responsible for committing into DVC
        :param auto_confirm: commit all changed files without confirmation
        """
        # In case something is added by user and not committed, we will take care of it
        commit_cmd = ["dvc", "commit", "-q"]
        if auto_confirm:
            commit_cmd.append("-f")
        execute_command(commit_cmd, capture_output=False)

    @staticmethod
    def push(remote: str) -> Any:
        push_cmd = ["dvc", "push"]
        if remote:
            push_cmd.append("-r")
            push_cmd.append(remote)
        execute_command(push_cmd, capture_output=False)



import os
from unittest.mock import patch

from fds.services.dvc_service import DvcChoices
from fds.utils import does_file_exist, execute_command, convert_bytes_to_string
from tests.it.helpers import IntegrationTestCase

class TestDvc(IntegrationTestCase):

    def test_init_dvc(self):
        self.git_service.init()
        self.dvc_service.init()
        assert does_file_exist(f"{self.repo_path}/.dvc") is True

    def test_init_git_already_exists(self):
        execute_command(["dvc", "init", "--no-scm"])
        assert does_file_exist(f"{self.repo_path}/.dvc") is True
        msg = self.dvc_service.init()
        assert msg == "DVC already initialized"
        assert does_file_exist(f"{self.repo_path}/.dvc") is True

    def test_init_dvc_failure(self):
        # Without git, dvc initialization fails without --no-scm
        self.assertRaises(Exception, self.dvc_service.init)

    def test_status(self):
        self.dvc_service.status()

    def test_add(self):
        self.git_service.init()
        self.dvc_service.init()
        super().create_fake_dvc_data()
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file" in convert_bytes_to_string(output.stdout)
        msg = self.dvc_service.add("large_file")
        assert does_file_exist(f"{self.repo_path}/large_file.dvc")
        assert msg == "DVC add successfully executed"

    def test_add_nothing(self):
        self.git_service.init()
        self.dvc_service.init()
        super().create_fake_dvc_data()
        msg = self.dvc_service.add(f"dvc_data/file-0")
        assert msg == "Nothing to add in DVC"

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.IGNORE.value})
    def test_add_check_ignore(self, get_choice):
        self.fds_service.init()
        super().create_fake_dvc_data()
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file" in convert_bytes_to_string(output.stdout)
        assert does_file_exist(f".gitignore") is False
        msg = self.dvc_service.add(".")
        assert does_file_exist(f".gitignore") is True
        assert msg == "Nothing to add in DVC"
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file" not in convert_bytes_to_string(output.stdout)

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_GIT.value})
    def test_add_check_add_git(self, get_choice):
        self.fds_service.init()
        super().create_fake_dvc_data()
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file" in convert_bytes_to_string(output.stdout)
        msg = self.dvc_service.add(".")
        assert msg == "Nothing to add in DVC"

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_DVC.value})
    def test_add_check_add_dvc(self, get_choice):
        self.fds_service.init()
        super().create_fake_dvc_data()
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file" in convert_bytes_to_string(output.stdout)
        msg = self.dvc_service.add(".")
        assert msg == "DVC add successfully executed"
        output = execute_command(["git", "status"], capture_output=True)
        assert f"large_file.dvc" in convert_bytes_to_string(output.stdout)

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_DVC.value})
    def test_commit_auto_confirm(self, get_choice):
        self.fds_service.init()
        super().create_fake_dvc_data()
        self.dvc_service.add(".")
        self.dvc_service.commit(True)
        execute_command(["git", "add", "large_file.dvc"], capture_output=True)
        output = execute_command(["dvc", "dag"], capture_output=True)
        assert "large_file.dvc" in convert_bytes_to_string(output.stdout)
        super().create_fake_dvc_data()
        self.dvc_service.commit(True)
        output = execute_command(["git", "diff", "--raw"], capture_output=True)
        assert "large_file.dvc" in convert_bytes_to_string(output.stdout)

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_DVC.value})
    def test_commit_no_auto_confirm(self, get_choice):
        self.fds_service.init()
        super().create_fake_dvc_data()
        self.dvc_service.add(".")
        self.dvc_service.commit(False)
        output = execute_command(["dvc", "dag"], capture_output=True)
        assert "large_file.dvc" in convert_bytes_to_string(output.stdout)



import os
from typing import Any

from fds.services.base_service import BaseService
from fds.services.pretty_print import PrettyPrint
from fds.utils import execute_command, convert_bytes_to_string, does_file_exist, check_git_ignore


class GitService(BaseService):
    """
    Git Service responsible for all the git commands of fds
    """
    def __init__(self):
        self.repo_path = os.path.curdir
        self.printer = PrettyPrint()

    def init(self) -> str:
        # Check if git is already initialized
        if does_file_exist(f"{self.repo_path}/.git"):
            return "git already initialized"
        execute_command(['git', 'init', self.repo_path])
        return "git initialized successfully"

    def status(self) -> Any:
        return execute_command(["git", "status"], capture_output=False)

    def add(self, add_argument: str) -> Any:
        git_output = check_git_ignore(add_argument)
        if convert_bytes_to_string(git_output.stdout) != '':
            return
        # This will take care of adding everything in the argument to add including the .dvc files inside it
        execute_command(["git", "add", add_argument])
        # Explicitly adding the .dvc file in the root because that wont be added by git
        dvc_file = f"{add_argument}.dvc"
        if does_file_exist(dvc_file):
            execute_command(["git", "add", f"{add_argument}.dvc"])
        ignore_file = ".gitignore"
        if does_file_exist(ignore_file):
            execute_command(["git", "add", ignore_file])

    def commit(self, message: str) -> Any:
        execute_command(["git", "commit", "-am", message], capture_output=False)

    @staticmethod
    def push(remote: str, ref: str) -> Any:
        push_cmd = ["git", "push"]
        if remote:
            push_cmd.append(remote)
            if ref:
                push_cmd.append(ref)
            else:
                check_curr_branch = execute_command(["git", "rev-parse", "--abbrev-ref", "HEAD"])
                curr_branch = convert_bytes_to_string(check_curr_branch.stdout).rstrip('\n')
                if curr_branch == '':
                    raise Exception("No git branch found to push to")
                push_cmd.append(curr_branch)
        execute_command(push_cmd, capture_output=False)



from fds.services.dvc_service import DVCService
from fds.services.git_service import GitService
from fds.services.pretty_print import PrettyPrint


class FdsService(object):
    """
    Fds Service responsible for all the commands of fds
    """
    # Todo: May be use dependency injection if required
    def __init__(
            self,
            git_service: GitService,
            dvc_service: DVCService
    ):
        self.git_service = git_service
        self.dvc_service = dvc_service
        self.printer = PrettyPrint()

    def init(self):
        """
        fds init
        """
        # Git init
        try:
            self.printer.success(self.git_service.init())
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("Git repo failed to initialize")
        # Dvc init
        try:
            self.printer.warn(self.dvc_service.init())
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("DVC repo failed to initialize")

    def status(self):
        """
        fds status
        """
        # Git status
        try:
            self.printer.success("========== Git repo status ==========")
            self.git_service.status()
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("Git status failed to execute")
        # Dvc status
        try:
            self.printer.warn("========== DVC repo status ==========")
            self.dvc_service.status()
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("DVC status failed to execute")

    def add(self, add_command: str):
        """
        fds add
        """
        # First let the user add files into dvc
        # Then remaining goes to git by default
        # Dvc add
        try:
            add_msg = self.dvc_service.add(add_command)
            self.printer.warn(add_msg)
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("DVC add failed to execute")
        # Add remaining to git
        try:
            self.git_service.add(add_command)
            self.printer.success("Git add successfully executed")
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("Git add failed to execute")

    def commit(self, message: str, yes: bool = True):
        try:
            self.dvc_service.commit(yes)
            self.printer.warn("Successfully committed to DVC")
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("DVC commit failed to execute")
        try:
            self.git_service.commit(message)
            self.printer.success("Successfully committed to Git")
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("Git commit failed to execute")

    def push(self, git_remote: str, dvc_remote: str, ref: str = None):
        """
        fds push
        """
        try:
            self.git_service.push(remote=git_remote, ref=ref)
            self.printer.success("Successfully pushed to Git remote")
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("Git push failed to execute")
        try:
            self.dvc_service.push(remote=dvc_remote)
            self.printer.warn("Successfully pushed to DVC remote")
        except Exception as e:
            self.printer.error(str(e))
            raise Exception("DVC push failed to execute")

    def save(self, message: str, git_remote: str, dvc_remote: str):
        self.add(".")
        self.commit(message)
        # TODO: add autodetect of remotes, ask users if they want to set a remote,
        #  and then push to default remotes, instead of manually entering remote names.
        self.push(git_remote, dvc_remote)
        self.printer.success("====================================")
        self.printer.success("Successfully saved current workspace")



import logging
import logging.config

logging.basicConfig(format='%(asctime)s at %(name)s.%(funcName)s'
                           ' (File "%(pathname)s", line %(lineno)d) %(levelname)s - %(message)s')


class Logger(object):

    logging_level = logging.INFO
    _logger = None

    @classmethod
    def set_logging_level(cls, level):
        cls.logging_level = level
        cls._logger = None

    @classmethod
    def get_logger(cls, logger_name: str):
        if cls._logger is None:
            # Ignoring the logger of dvc, by setting it to highest level
            logging.getLogger('dvc').setLevel(logging.CRITICAL)

            # Logger for fds
            cls._logger = logging.getLogger(logger_name)
            cls._logger.setLevel(cls.logging_level)

        return cls._logger



from typing import Any


class BaseService(object):
    """
    Base Service responsible for all the commands of fds
    """
    def init(self) -> str:
        """
        The init command
        :return: Success message
        """
        pass

    def status(self) -> Any:
        """
        The status command
        :return: Nothing
        """
        pass

    def add(self, add_argument: str) -> Any:
        """
        The add command
        :return: Nothing
        """
        pass

    def commit(self, message: str) -> Any:
        """
        The commit command
        @param message: Commit message
        :return: Nothing
        """
        pass



import os
from unittest.mock import patch

from fds.services.dvc_service import DvcChoices
from fds.utils import does_file_exist, execute_command, convert_bytes_to_string
from tests.it.helpers import IntegrationTestCase


class TestFds(IntegrationTestCase):

    def test_init_success(self):
        self.fds_service.init()
        assert does_file_exist(f"{self.repo_path}/.git")
        assert does_file_exist(f"{self.repo_path}/.dvc")

    def test_status(self):
        self.fds_service.init()
        self.fds_service.status()

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_DVC.value})
    def test_add(self, get_choice):
        self.fds_service.init()
        super().create_fake_git_data()
        super().create_fake_dvc_data()
        self.fds_service.add(".")
        output = execute_command(["git", "status"], capture_output=True)
        # Check DVC add
        assert f"new file:   large_file.dvc" in convert_bytes_to_string(output.stdout)
        # Check Git add
        assert f"new file:   git_data/file-0" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-1" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-2" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-3" in convert_bytes_to_string(output.stdout)
        assert f"new file:   git_data/file-4" in convert_bytes_to_string(output.stdout)

    @patch("fds.services.dvc_service.DVCService._get_choice", return_value={"selection_choice": DvcChoices.ADD_TO_DVC.value})
    def test_commit(self, get_choice):
        self.fds_service.init()
        super().create_fake_git_data()
        super().create_fake_dvc_data()
        self.fds_service.add(".")
        self.fds_service.commit("Commit 1", True)
        output = execute_command(["git", "log", "--oneline"], capture_output=True)
        assert "Commit 1" in convert_bytes_to_string(output.stdout)
        output = execute_command(["dvc", "dag"], capture_output=True)
        assert "large_file.dvc" in convert_bytes_to_string(output.stdout)
        super().create_fake_dvc_data()
        self.fds_service.commit("Commit 2", True)
        output = execute_command(["git", "log", "--oneline"], capture_output=True)
        assert "Commit 2" in convert_bytes_to_string(output.stdout)
        output = execute_command(["git", "diff", "--raw", "HEAD~1"], capture_output=True)
        assert "large_file.dvc" in convert_bytes_to_string(output.stdout)

    def test_commit_git(self):
        self.fds_service.init()
        super().create_fake_git_data()
        self.fds_service.add(".")
        self.fds_service.commit("Commit 1", False)
        output = execute_command(["git", "log", "--oneline"], capture_output=True)
        assert "Commit 1" in convert_bytes_to_string(output.stdout)




import setuptools

try:
    with open('README.md', 'r') as readme:
        long_description = readme.read()
except Exception:
    long_description = None

with open('requirements.txt') as f:
    requirements = f.readlines()

version = "0.1.7"

setuptools.setup(
    name="fastds",
    version=version,
    author="DAGsHub",
    license='MIT License',
    author_email="contact@dagshub.com",
    description="Command line wrapper for git and dvc",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/DAGsHub/fds",
    packages=setuptools.find_packages(exclude=("tests",)),
    entry_points={
        'console_scripts': [
            'fds=fds.cli:main',
            'sdf=fds.cli:main'
        ]
    },
    install_requires=requirements,
    zip_safe=False,
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6'
)



from enum import Enum


# Commands supported by ff
class Commands(Enum):
    INIT = "init"
    STATUS = "status"
    ADD = "add"
    COMMIT = "commit"
    PUSH = "push"
    SAVE = "save"

class AddCommands(Enum):
    ALL = "."



import os
import shutil
import unittest
import tempfile
from pathlib import Path

from fds.services.dvc_service import DVCService
from fds.services.fds_service import FdsService
from fds.services.git_service import GitService


class IntegrationTestCase(unittest.TestCase):

    def setUp(self):
        super().setUp()
        self.repo_path = tempfile.mkdtemp()
        self.git_service = GitService()
        self.dvc_service = DVCService()
        self.fds_service = FdsService(self.git_service, self.dvc_service)
        os.chdir(self.repo_path)

    def tearDown(self):
        super().tearDown()
        shutil.rmtree(self.repo_path)

    def create_fake_git_data(self):
        git_path = f"{self.repo_path}/git_data"
        Path(git_path).mkdir(parents=True, exist_ok=True)
        # Creating 5 random files
        for i in range(0,5):
            self.create_dummy_file(f"{git_path}/file-{i}", 10)

    def create_dummy_file(self, file_name: str, size: int):
        with open(file_name, 'wb') as fout:
            fout.write(os.urandom(size))

    def create_fake_dvc_data(self):
        dvc_path = f"{self.repo_path}/dvc_data"
        Path(dvc_path).mkdir(parents=True, exist_ok=True)
        # creating a big folder
        for i in range(0,101):
            self.create_dummy_file(f"{dvc_path}/file-{i}", 10)
        # creating one large file
        self.create_dummy_file(f"large_file", 11 * 1024)



from colorama import init
from colorama import Fore

class PrettyPrint(object):

    def __init__(self):
        init(autoreset=True)

    def warn(self, text: str):
        print(Fore.YELLOW + text)

    def log(self, text: str):
        print(Fore.reset + text)

    def error(self, text: str):
        print(Fore.RED + text)

    def success(self, text: str):
        print(Fore.GREEN + text)

    def convert_bytes_to_str(self, bytes_data: bytes) -> str:
        return bytes_data.decode("utf-8")



from shutil import which
import PyInquirer

from fds.domain.commands import Commands
from fds.logger import Logger
from fds.services.dvc_service import DVCService
from fds.services.fds_service import FdsService
from fds.services.git_service import GitService
from fds.services.pretty_print import PrettyPrint
from fds.utils import execute_command


class Run(object):
    def __init__(self, arguments: dict):
        self.logger = Logger.get_logger("fds.Run")
        self.arguments = arguments
        self.service = FdsService(GitService(), DVCService())
        self.printer = PrettyPrint()

    def pre_execute_hook(self):
        # Check if dvc is installed
        ret_code = 0
        if which("dvc") is None:
            ret_code = 1
            self.printer.error("dvc executable is not installed or found")
            questions = [
                {
                    'type': 'confirm',
                    'message': 'Should we install dvc [https://dvc.org/] for you right now?\n' +
                               '  Will install using `pip install dvc<3`',
                    'name': 'install',
                    'default': False,
                },
            ]
            answers = PyInquirer.prompt(questions)
            if answers["install"]:
                execute_command(["pip install 'dvc<3'"], shell=True, capture_output=False)
                ret_code = 0
            else:
                # Provide instructions
                self.printer.warn("You can install dvc manually from https://dvc.org/doc/install")

        if which("git") is None:
            self.printer.error("git executable is not found, please install git from https://git-scm.com/downloads")
            ret_code |= 2

        return ret_code

    def execute(self):
        # Run pre execute hook
        hook_ret_code = self.pre_execute_hook()
        if hook_ret_code != 0:
            return hook_ret_code

        arguments = self.arguments
        self.logger.debug(f"arguments passed: {arguments}")
        if arguments["command"] == Commands.INIT.value:
            # Run init command stuff
            self.service.init()
            return 0
        elif arguments["command"] == Commands.STATUS.value:
            # Run status command stuff
            self.service.status()
            return 0
        elif arguments["command"] == Commands.ADD.value:
            # Run add command stuff
            self.service.add(arguments["add_command"])
            return 0
        elif arguments["command"] == Commands.COMMIT.value:
            if len(arguments.get("message", [])) == 1:
                message = arguments["message"][0]
            elif len(arguments.get("m", [])) == 1:
                message = arguments["m"][0]
            else:
                raise Exception("Enter a valid commit message")
            # Run commit command stuff
            self.service.commit(message, arguments['yes'])
            return 0
        elif arguments["command"] == Commands.PUSH.value:
            # Run push command stuff
            self.service.push(arguments["git_remote"], arguments["dvc_remote"], arguments["branch"])
            return 0
        elif arguments["command"] == Commands.SAVE.value:
            # Run save command stuff
            self.service.save(arguments["message"], arguments["git_remote"], arguments["dvc_remote"])
            return 0
        else:
            raise Exception("Invalid operation")



import argparse
import logging
import sys

from fds.logger import Logger

# Argument parser stuff
from fds.run import Run
from fds.services.pretty_print import PrettyPrint

arg_parser = argparse.ArgumentParser(description="One command for all your git and dvc needs",
                                     prog="fds")
# Command choice
command_subparser = arg_parser.add_subparsers(dest="command", help="command (refer commands section in documentation)")

# init
parser_init = command_subparser.add_parser('init', help='initialize a git and dvc repository')

# status
parser_status = command_subparser.add_parser('status', help='get status of your git and dvc repository')

# add
parser_add = command_subparser.add_parser('add', help='add files/folders to git and dvc repository')
parser_add.add_argument('add_command', help="choose what to add using . will add everything")

# COMMIT
parser_commit = command_subparser.add_parser('commit', help='commits added changes to git and dvc repository',
                                             conflict_handler='resolve')
parser_commit.add_argument('-y', "--yes",
                           help="Don't ask for confirmation for committing file changes",
                           action="store_true", default=False)
parser_commit_msg_grp = parser_commit.add_mutually_exclusive_group()
parser_commit_msg_grp.add_argument('message', nargs='*', help="commit message", default='')
parser_commit_msg_grp.add_argument('-m', nargs=1, help="commit message", default='')

# push
parser_push = command_subparser.add_parser(
    'push',
    help='push commits to remote git and dvc repository'
)
parser_push.add_argument('-gr', '--git-remote', help="git remote name, default 'origin'", default="origin")
parser_push.add_argument('-dr', '--dvc-remote', help="dvc remote name, default 'origin'", default="origin")
parser_push.add_argument(
    'branch',
    help="A git branch that you want to push to (accepts refspec), defaults to current branch",
    nargs='?'
)
# save
parser_save = command_subparser.add_parser(
    'save',
    help='saves all project files to a new version and pushes them to your remote'
)
parser_save.add_argument('-gr', '--git-remote', help="git remote name, default 'origin'", default="origin")
parser_save.add_argument('-dr', '--dvc-remote', help="dvc remote name, default 'origin'", default="origin")
parser_save.add_argument('message', help="save message")

# argument for log level
arg_parser.add_argument("-v", "--verbose", help="set log level to DEBUG",
                        action="store_true", default=False)


def parse_args(args):
    arguments = vars(arg_parser.parse_args(args=args or ["--help"]))
    return arguments


def main(args=None):
    if args is None:
        args = sys.argv[1:]
    parsed_args = parse_args(args=args)
    printer = PrettyPrint()
    if bool(parsed_args["verbose"]):
        Logger.set_logging_level(logging.DEBUG)
    try:
        result = Run(arguments=parsed_args).execute()
    except Exception as e:
        printer.error(str(e))
        result = 1
    sys.exit(result)


if __name__ == "__main__":
    main()
